@@ -9,6 +9,26 @@
 -- Convenient creation of command-line-driven executables for
 -- rendering diagrams using the cairo backend.
 --
+-- * 'defaultMain' creates an executable which can render a single
+--   diagram at various options.
+--
+-- * 'multiMain' is like 'defaultMain' but allows for a list of
+--   diagrams from which the user can choose one to render.
+--
+-- * 'animMain' is like 'defaultMain' but for animations instead of
+--   diagrams.
+--
+-- If you want to generate diagrams programmatically---/i.e./ if you
+-- want to do anything more complex than what the below functions
+-- provide---you have several options.
+--
+-- * A simple but somewhat inflexible approach is to wrap up
+--   'defaultMain' (or 'multiMain', or 'animMain') in a call to
+--   'System.Environment.withArgs'.
+--
+-- * A more flexible approach is to directly call 'renderDia'; see
+--   "Diagrams.Backend.Cairo" for more information.
+--
 -----------------------------------------------------------------------------
 
 module Diagrams.Backend.Cairo.CmdLine
@@ -19,6 +39,7 @@ module Diagrams.Backend.Cairo.CmdLine
        , Cairo
        ) where
 
+import Data.List (intercalate)
 import Diagrams.Prelude hiding (width, height, interval)
 import Diagrams.Backend.Cairo
 
@@ -33,7 +54,7 @@ import System.Console.CmdArgs.Implicit hiding (args)
 import Prelude hiding      (catch)
 
 import Data.Maybe          (fromMaybe)
-import Control.Monad       (when, forM_)
+import Control.Monad       (when, forM_, mplus)
 import Data.List.Split
 
 import Text.Printf
@@ -45,18 +66,29 @@ import System.Process      (runProcess, waitForProcess)
 import System.IO           (openFile, hClose, IOMode(..),
                             hSetBuffering, BufferMode(..), stdout)
 import System.Exit         (ExitCode(..))
-import System.Time         (ClockTime, getClockTime)
 import Control.Concurrent  (threadDelay)
 import Control.Exception   (catch, SomeException(..), bracket)
 
 #ifdef CMDLINELOOP
 import System.Posix.Process (executeFile)
+#if MIN_VERSION_directory(1,2,0)
+import Data.Time.Clock (UTCTime,getCurrentTime)
+type ModuleTime = UTCTime
+getModuleTime :: IO  ModuleTime
+getModuleTime = getCurrentTime
+#else
+import System.Time         (ClockTime, getClockTime)
+type ModuleTime = ClockTime
+getModuleTime :: IO  ModuleTime
+getModuleTime = getClockTime
+#endif
 #endif
 
 data DiagramOpts = DiagramOpts
                    { width     :: Maybe Int
                    , height    :: Maybe Int
                    , output    :: FilePath
+                   , list      :: Bool
                    , selection :: Maybe String
                    , fpu       :: Double
 #ifdef CMDLINELOOP
@@ -85,6 +117,9 @@ diagramOpts prog sel = DiagramOpts
               &= help "Name of the diagram to render"
               &= (if sel then typ "NAME" else ignore)
 
+  , list = def
+         &= (if sel then help "List all available diagrams" else ignore)
+
   , fpu = 30
           &= typ "FLOAT"
           &= help "Frames per unit time (for animations)"
@@ -104,20 +139,52 @@ diagramOpts prog sel = DiagramOpts
 -- | This is the simplest way to render diagrams, and is intended to
 --   be used like so:
 --
--- > ... definitions ...
+-- > ... other definitions ...
+-- > myDiagram = ...
 -- >
 -- > main = defaultMain myDiagram
 --
---   Compiling this file will result in an executable which takes
---   various command-line options for setting the size, output file,
---   and so on, and renders @myDiagram@ with the specified options.
+--   Compiling a source file like the above example will result in an
+--   executable which takes command-line options for setting the size,
+--   output file, and so on, and renders @myDiagram@ with the
+--   specified options.
 --
 --   On Unix systems, the generated executable also supports a
 --   rudimentary \"looped\" mode, which watches the source file for
 --   changes and recompiles itself on the fly.
 --
 --   Pass @--help@ to the generated executable to see all available
---   options.
+--   options.  Currently it looks something like
+--
+-- @
+-- Command-line diagram generation.
+--
+-- Foo [OPTIONS]
+--
+-- Common flags:
+--   -w --width=INT         Desired width of the output image
+--   -h --height=INT        Desired height of the output image
+--   -o --output=FILE       Output file
+--   -f --fpu=FLOAT         Frames per unit time (for animations)
+--   -l --loop              Run in a self-recompiling loop
+--   -s --src=FILE          Source file to watch
+--   -i --interval=SECONDS  When running in a loop, check for changes every n
+--                          seconds.
+--   -? --help              Display help message
+--   -V --version           Print version information
+-- @
+--
+--   For example, a couple common scenarios include
+--
+-- @
+-- $ ghc --make MyDiagram
+--
+--   # output image.png with a width of 400px (and auto-determined height)
+-- $ ./MyDiagram -o image.png -w 400
+--
+--   # output 200x200 dia.pdf, then watch for changes every 10 seconds
+-- $ ./MyDiagram -o dia.pdf -h 200 -w 200 -l -i 10
+-- @
 
 defaultMain :: Diagram Cairo R2 -> IO ()
 defaultMain d = do
@@ -140,49 +207,74 @@ chooseRender opts d =
                  "pdf" -> PDF
                  "svg" -> SVG
                  _     -> PDF
-               sizeSpec = case (width opts, height opts) of
-                            (Nothing, Nothing) -> Absolute
-                            (Just w, Nothing)  -> Width (fromIntegral w)
-                            (Nothing, Just h)  -> Height (fromIntegral h)
-                            (Just w, Just h)   -> Dims (fromIntegral w)
-                                                       (fromIntegral h)
-           fst $ renderDia Cairo (CairoOptions
-                                    (output opts)
-                                    sizeSpec
-                                    outTy
-                                 )
-                           d
+           fst $ renderDia
+                   Cairo
+                   ( CairoOptions
+                     (output opts)
+                     (mkSizeSpec
+                       (fromIntegral <$> width opts)
+                       (fromIntegral <$> height opts)
+                     )
+                     outTy
+                     False
+                   )
+                   d
        | otherwise -> putStrLn $ "Unknown file type: " ++ last ps
 
 -- | @multiMain@ is like 'defaultMain', except instead of a single
 --   diagram it takes a list of diagrams paired with names as input.
---   The generated executable then takes an argument specifying the
---   name of the diagram that should be rendered.  This is a
---   convenient way to create an executable that can render many
---   different diagrams without modifying the source code in between
---   each one.
+--   The generated executable then takes a @--selection@ option
+--   specifying the name of the diagram that should be rendered.  The
+--   list of available diagrams may also be printed by passing the
+--   option @--list@.
+--
+--   Example usage:
+--
+-- @
+-- $ ghc --make MultiTest
+-- [1 of 1] Compiling Main             ( MultiTest.hs, MultiTest.o )
+-- Linking MultiTest ...
+-- $ ./MultiTest --list
+-- Available diagrams:
+--   foo bar
+-- $ ./MultiTest --selection bar -o Bar.png -w 200
+-- @
+
 multiMain :: [(String, Diagram Cairo R2)] -> IO ()
 multiMain ds = do
   prog <- getProgName
   opts <- cmdArgs (diagramOpts prog True)
-  case selection opts of
-    Nothing  -> putStrLn "No diagram selected."
-    Just sel -> case lookup sel ds of
-      Nothing -> putStrLn $ "Unknown diagram: " ++ sel
-      Just d  -> chooseRender opts d
+  if list opts
+    then showDiaList (map fst ds)
+    else
+      case selection opts of
+        Nothing  -> putStrLn "No diagram selected." >> showDiaList (map fst ds)
+        Just sel -> case lookup sel ds of
+          Nothing -> putStrLn $ "Unknown diagram: " ++ sel
+          Just d  -> chooseRender opts d
 
--- | @animMain@ takes an animation and produces a command-line program
---   which will crudely \"render\" the animation by rendering one image
---   for each frame, named by extending the given output file name by
---   consecutive integers.  For example if the given output file name
---   is @foo\/blah.png@, the frames will be saved in @foo\/blah001.png@,
---   @foo\/blah002.png@, and so on (the number of padding digits used
---   depends on the total number of frames).  It is up to the user to
---   take these images and stitch them together into an actual
---   animation format (using, /e.g./ @ffmpeg@).
+-- | Display the list of diagrams available for rendering.
+showDiaList :: [String] -> IO ()
+showDiaList ds = do
+  putStrLn "Available diagrams:"
+  putStrLn $ "  " ++ intercalate " " ds
+
+-- | @animMain@ is like 'defaultMain', but renders an animation
+-- instead of a diagram.  It takes as input an animation and produces
+-- a command-line program which will crudely \"render\" the animation
+-- by rendering one image for each frame, named by extending the given
+-- output file name by consecutive integers.  For example if the given
+-- output file name is @foo\/blah.png@, the frames will be saved in
+-- @foo\/blah001.png@, @foo\/blah002.png@, and so on (the number of
+-- padding digits used depends on the total number of frames).  It is
+-- up to the user to take these images and stitch them together into
+-- an actual animation format (using, /e.g./ @ffmpeg@).
 --
 --   Of course, this is a rather crude method of rendering animations;
 --   more sophisticated methods will likely be added in the future.
+--
+-- The @--fpu@ option can be used to control how many frames will be
+-- output for each second (unit time) of animation.
 animMain :: Animation Cairo R2 -> IO ()
 animMain anim = do
   prog <- getProgName
@@ -202,7 +294,7 @@ indexize nDigits i opts = opts { output = output' }
         (base, ext) = splitExtension (output opts)
 
 #ifdef CMDLINELOOP
-waitForChange :: Maybe ClockTime -> DiagramOpts -> String -> [String] -> IO ()
+waitForChange :: Maybe ModuleTime -> DiagramOpts -> String -> [String] -> IO ()
 waitForChange lastAttempt opts prog args = do
     hSetBuffering stdout NoBuffering
     go lastAttempt
@@ -212,7 +304,7 @@ waitForChange lastAttempt opts prog args = do
           (newBin, newAttempt) <- recompile lastAtt prog (src opts)
           if newBin
             then executeFile prog False args Nothing
-            else go $ getFirst (First newAttempt <> First lastAtt)
+            else go $ newAttempt `mplus` lastAtt
 
 -- | @recompile t prog@ attempts to recompile @prog@, assuming the
 --   last attempt was made at time @t@.  If @t@ is @Nothing@ assume
@@ -222,7 +314,7 @@ waitForChange lastAttempt opts prog args = do
 --   of this attempt.  Otherwise (if nothing has changed since the
 --   last attempt), return @Nothing@.  Also return a Bool saying
 --   whether a successful recompilation happened.
-recompile :: Maybe ClockTime -> String -> Maybe String -> IO (Bool, Maybe ClockTime)
+recompile :: Maybe ModuleTime -> String -> Maybe String -> IO (Bool, Maybe ModuleTime)
 recompile lastAttempt prog mSrc = do
   let errFile = prog ++ ".errors"
       srcFile = fromMaybe (prog ++ ".hs") mSrc
@@ -239,11 +331,11 @@ recompile lastAttempt prog mSrc = do
         then putStrLn "" >> putStrLn (replicate 75 '-') >> readFile errFile >>= putStr
         else putStrLn "done."
 
-      curTime <- getClockTime
+      curTime <- getModuleTime
       return (status == ExitSuccess, Just curTime)
 
     else return (False, Nothing)
 
  where getModTime f = catch (Just <$> getModificationTime f)
                             (\(SomeException _) -> return Nothing)
-#endif
\ No newline at end of file
+#endif
